body {
	background-color: #000;
}

.row {
	margin-left: 0px;
	margin-right: 0px;
}
.opacer {
	background-color: #f5f5dc61;
	padding: 0px 0px 24px 24px;
	border-radius: 10px 0px 0px 10px;
	margin-right: -25px;
}
.bg-img {
	position: absolute;
	width: 100%;
	height: 100%;
	opacity: 1;
	filter: alpha(opacity=100);
    top: 0px;
    left: 0px;
}
.content {
	font-family: Avenir,Helvetica,sans-serif;
	color: #141414;
	background-color: none;
	z-index: 2;
	position: absolute;
	top: 50%;
    right: 10px;
	-webkit-transform: translateY(-50%);
	-ms-transform: translateY(-50%);
	transform: translateY(-50%);
}
a, a:visited {
	color: #20201D;
}

a:hover {
	color: #464640;
}
h1 {
	font-size: 200px;
	margin-bottom: 0;
	margin-top: 0;
	color: #ffffff;
}
h1 .small {
	font-size: 20px;
	color: #20201D;
}
h2 {
	margin-top: 0;
	font-size: 24px;
	color: #1F1F1E;
}
.search {
	position: relative;
}
.search input {
	border: 1px solid #2b2b2b;
	min-width: 100px;
	height: 52px;
	padding: 11px 30px 12px 30px;
	border-radius: 40px;
	-webkit-appearance: none;
	-moz-appearance: none;
	-webkit-transition: all 0.3s ease-in-out;
	transition: all 0.3s ease-in-out;
	background: transparent;
	color: #141414;
}
.search input:focus {
	min-width: 350px;
}
.search button {
	width: 40px;
	height: 52px;
	position: absolute;
	right: 5px;
	border: none;
	background: transparent;
}
.search button i {
	color: #141414;
	font-size: 22px;
	line-height: 20px;
}
.search :focus {
	outline-color: transparent;
	outline-style: none;
}
.btn {
	display: inline-block;
	border: 1px solid #2b2b2b;
	border-radius: 40px;
	padding: 15px 30px;
	margin-right: 15px;
	color: #141414;
}
.btn:hover {
	color: #e2e2e2;
	background: rgba(255, 255, 255, 0.1);
}
p {
	margin: 0px 30px 30px 30px;;
}
